# -*- coding: utf-8 -*-
"""
Created on Mon Feb 26 11:23:00 2018

@author: eivin

This script will make the model and prepare the jobs
"""
from abaqus import *
from abaqusConstants import *
import __main__

import math
import section
import regionToolset
import displayGroupMdbToolset as dgm
import part
import material
import assembly
import step
import interaction
import load
import mesh
import optimization
import job
import sketch
import visualization
import xyPlot
import displayGroupOdbToolset as dgo
import connectorBehavior
import sys
import os

#def makeModel(modelName,dim1,dim2,dim3,dim4,thickness):
def makeModel(modelName,a,b,c,d,thetaDeg,thickness):    
    theModel = mdb.Model(name='Model-1', modelType=STANDARD_EXPLICIT)
    
    theta = math.radians(thetaDeg)
    
    bx=b*math.cos(theta)
    by=b*math.sin(theta)
    cx=c*math.sin(theta)
    cy=c*math.cos(theta)
    cxy=(c-c/math.cos(theta))/(-math.tan(theta))
    d2=d/2
    dxy=(d2-d/math.cos(theta))/(-math.tan(theta))
    dx=d*math.sin(theta)
    dy=d*math.cos(theta)
    w=a+bx+dxy+cxy+dx+a
    h=by+dy+c+d/2.0
    
    # CREATING THE PART:
    
    s = theModel.ConstrainedSketch(name='__profile__', 
        sheetSize=100.0)
    g, v, d, c = s.geometry, s.vertices, s.dimensions, s.constraints
    s.setPrimaryObject(option=STANDALONE)
    # Note: no constraints or dimensions on the sketch needed, just the lines: 
    s.setPrimaryObject(option=STANDALONE)
    
    s.Line(point1=(0.0,0.0), point2=(a+bx,0.0)) #0-1
    s.Line(point1=(a+bx,0.0), point2=(a,by)) #2
    s.Line(point1=(a,by), point2=(a+dx,by+dy))#3
    s.Line(point1=(a+dx,by+dy), point2=(a+bx+dxy,d2))#4
    s.Line(point1=(a+bx+dxy,d2), point2=(w,d2))#5
    s.Line(point1=(w,d2), point2=(w,h))#6
    s.Line(point1=(w,h), point2=(w-a-bx,h))#7
    s.Line(point1=(w-a-bx,h), point2=(w-a,h-by))#8
    s.Line(point1=(w-a,h-by), point2=(w-a-dx,h-by-dy))#9
    s.Line(point1=(w-a-dx,h-by-dy), point2=(w-a-bx-dxy,h-d2))#10
    s.Line(point1=(w-a-bx-dxy,h-d2), point2=(0.0,h-d2))#11
    s.Line(point1=(0.0,h-d2), point2=(0.0,0.0))#11-0
    
    p = theModel.Part(name='BasePart', dimensionality=THREE_D, 
        type=DEFORMABLE_BODY)
    p.BaseSolidExtrude(sketch=s, depth=thickness)
    s.unsetPrimaryObject()
    session.viewports['Viewport: 1'].setValues(displayedObject=p)
    del theModel.sketches['__profile__']
    
    
#    s = theModel.ConstrainedSketch(name='__profile__', 
#        sheetSize=100.0)
#    g, v, d, c = s.geometry, s.vertices, s.dimensions, s.constraints
#    s.setPrimaryObject(option=STANDALONE)
#    # Note: no constraints or dimensions on the sketch needed, just the lines: 
#    s.Line(point1=(0.0, 0.0), point2=(0.0, dim2+dim3))
#    s.Line(point1=(0.0, dim2+dim3), point2=(dim4, dim2+dim3))
#    s.Line(point1=(dim4, dim2+dim3), point2=(dim4, dim2+dim3+dim2))
#    s.Line(point1=(dim4, dim2+dim3+dim2), point2=(dim4+dim1, dim2+dim3+dim2))
#    s.Line(point1=(dim4+dim1, dim2+dim3+dim2), point2=(dim4+dim1, dim2))
#    s.Line(point1=(dim4+dim1, dim2), point2=(dim1, dim2))
#    s.Line(point1=(dim1, dim2), point2=(dim1, 0.0))
#    s.Line(point1=(dim1, 0.0), point2=(0.0, 0.0))
#    p = theModel.Part(name='BasePart', dimensionality=THREE_D, 
#        type=DEFORMABLE_BODY)
#    p.BaseSolidExtrude(sketch=s, depth=thickness)
#    s.unsetPrimaryObject()
#    del theModel.sketches['__profile__']

def copyBasePartToNewModel(modelName):
    # Non-linear
    mdb.Model(name=modelName, modelType=STANDARD_EXPLICIT)
    mdb.models[modelName].Part('BasePart', mdb.models['Model-1'].parts['BasePart'])
    
    # Linear
    modelNameL = modelName + '-L'
    mdb.Model(name=modelNameL, modelType=STANDARD_EXPLICIT)
    mdb.models[modelNameL].Part('BasePartL', mdb.models['Model-1'].parts['BasePart'])


def meshAndMirror(modelName,esize):
    # Non-linear
    m=mdb.models[modelName]
    p=m.parts['BasePart']
    p.seedPart(size=esize, deviationFactor=0.1, minSizeFactor=0.1)
    # Import mesh
    elemType1 = mesh.ElemType(elemCode=C3D20R, elemLibrary=STANDARD)
#    elemType2 = mesh.ElemType(elemCode=C3D15, elemLibrary=STANDARD)
#    elemType3 = mesh.ElemType(elemCode=C3D10, elemLibrary=STANDARD)
    p.Set(cells=p.cells, name='Set-1')
    p.setElementType(regions=p.sets['Set-1'], elemTypes=(elemType1,))
    p.setMeshControls(regions=p.cells, technique=SWEEP)
    p.generateMesh()
    p.PartFromMesh(name='MeshPart1', copySets=True)    
    p1 = m.parts['MeshPart1']
    p2 = m.Part(name='MeshPart1-Copy', 
        objectToCopy=p1, compressFeatureList=ON, mirrorPlane=XYPLANE)
    a = m.rootAssembly
    a.DatumCsysByDefault(CARTESIAN)
    inst1=a.Instance(name='MeshPart1-1', part=p1, dependent=ON)
    inst2=a.Instance(name='MeshPart1-Copy-1', part=p2, dependent=ON)
    inst = a.InstanceFromBooleanMerge(name='MeshPart2', instances=(
           inst1, inst2, ), 
           mergeNodes=BOUNDARY_ONLY, nodeMergingTolerance=1e-06, domain=MESH, 
           originalInstances=DELETE) 
    
    # Linear
    modelNameL = modelName + '-L'
    m=mdb.models[modelNameL]
    p=m.parts['BasePartL']
    p.seedPart(size=esize, deviationFactor=0.1, minSizeFactor=0.1)
    # Import mesh
    elemType1 = mesh.ElemType(elemCode=C3D20R, elemLibrary=STANDARD)
#    elemType2 = mesh.ElemType(elemCode=C3D15, elemLibrary=STANDARD)
#    elemType3 = mesh.ElemType(elemCode=C3D10, elemLibrary=STANDARD)
    p.Set(cells=p.cells, name='Set-1')
    p.setElementType(regions=p.sets['Set-1'], elemTypes=(elemType1,))
    p.setMeshControls(regions=p.cells, technique=SWEEP)
    p.generateMesh()
    p.PartFromMesh(name='MeshPart1', copySets=True)    
    p1 = m.parts['MeshPart1']
    p2 = m.Part(name='MeshPart1-Copy', 
        objectToCopy=p1, compressFeatureList=ON, mirrorPlane=XYPLANE)
    a = m.rootAssembly
    a.DatumCsysByDefault(CARTESIAN)
    inst1=a.Instance(name='MeshPart1-1', part=p1, dependent=ON)
    inst2=a.Instance(name='MeshPart1-Copy-1', part=p2, dependent=ON)
    inst = a.InstanceFromBooleanMerge(name='MeshPart2', instances=(
           inst1, inst2, ), 
           mergeNodes=BOUNDARY_ONLY, nodeMergingTolerance=1e-06, domain=MESH, 
           originalInstances=DELETE)


def materialAndProperties(modelName,Emod,Pois):
    # Non-linear
    m=mdb.models[modelName]
    m.Material(name='Material-1')
    m.materials['Material-1'].Elastic(table=((Emod, Pois), ))
    m.HomogeneousSolidSection(name='Section-1', material='Material-1', thickness=None)
    p = m.parts['MeshPart2']
    reg = p.Set(elements=p.elements, name='SET-ELEMENTS-ALL')
    p.SectionAssignment(region=reg, sectionName='Section-1', offset=0.0, 
        offsetType=MIDDLE_SURFACE, offsetField='', 
        thicknessAssignment=FROM_SECTION)
    
    # Linear
    modelNameL = modelName + '-L'
    m=mdb.models[modelNameL]
    m.Material(name='Material-1')
    m.materials['Material-1'].Elastic(table=((Emod, Pois), ))
    m.HomogeneousSolidSection(name='Section-1', material='Material-1', thickness=None)
    p = m.parts['MeshPart2']
    reg = p.Set(elements=p.elements, name='SET-ELEMENTS-ALL')
    p.SectionAssignment(region=reg, sectionName='Section-1', offset=0.0, 
        offsetType=MIDDLE_SURFACE, offsetField='', 
        thicknessAssignment=FROM_SECTION)


def getDimensions(modelName):
    # The objective is to find the max (and min) extension of the model
    # in all 3 directions, x,y and z. In that sense, the dimensions do not
    # need to be provided into the main functions parameter. There are probably
    # several methods to achive this goal. The following approach is rather 
    # straigth forward, alltough probably not the most computational efficient:
    xmin,ymin,zmin,xmax,ymax,zmax = 0.0, 0.0, 0.0, 0.0, 0.0, 0.0
    a = mdb.models[modelName].rootAssembly  
    allNodes = a.instances['MeshPart2-1'].nodes
    for n in allNodes:
        x,y,z = n.coordinates[0], n.coordinates[1], n.coordinates[2]
        xmin = min(xmin,x)
        ymin = min(ymin,y)
        zmin = min(zmin,z)
        xmax = max(xmax,x)
        ymax = max(ymax,y)
        zmax = max(zmax,z)
    return (xmin, ymin, zmin, xmax, ymax, zmax)
        

def createAssemblySets(modelName,dX,dY,th):
    # Non-linear
    elementSize=mdb.models[modelName].parts['BasePart'].getPartSeeds(SIZE)
    tol=0.1*elementSize
    inf=max(dX+tol,dY+tol,2*th+tol)
    a = mdb.models[modelName].rootAssembly  
    allNodes = a.instances['MeshPart2-1'].nodes
    nodesXa = allNodes.getByBoundingBox(0-tol, -inf, -inf, 0+tol, inf, inf)
    a.Set(nodes=nodesXa, name='SET-NODES-XA')
    nodesXb = allNodes.getByBoundingBox(dX-tol, -inf, -inf, dX+tol, inf, inf)
    a.Set(nodes=nodesXb, name='SET-NODES-XB')    
    nodesYa = allNodes.getByBoundingBox(-inf, 0-tol, -inf, inf, 0+tol, inf)
    a.Set(nodes=nodesYa, name='SET-NODES-YA')
    nodesYb = allNodes.getByBoundingBox(-inf, dY-tol, -inf, inf, dY+tol, inf)
    a.Set(nodes=nodesYb, name='SET-NODES-YB')
    nodeOrigo=allNodes.getByBoundingBox(0-tol, 0-tol, 0-tol, 0+tol, 0+tol, 0+tol)
    a.Set(nodes=nodeOrigo, name='SET-NODE-ORIGO')
    nodeSubtract=allNodes.getByBoundingBox(dX-tol, dY-tol, 0-tol, dX+tol, dY+tol, 0+tol)
    a.Set(nodes=nodeSubtract, name='SET-NODE-SUB')
    a.SetByBoolean(name='SET-SUB-XB',sets=[a.sets['SET-NODES-XB'],a.sets['SET-NODE-SUB']],operation=DIFFERENCE)
    
    # Linear
    modelNameL = modelName + '-L'
    elementSize=mdb.models[modelNameL].parts['BasePartL'].getPartSeeds(SIZE)
    tol=0.1*elementSize
    inf=max(dX+tol,dY+tol,2*th+tol)
    a = mdb.models[modelNameL].rootAssembly  
    allNodes = a.instances['MeshPart2-1'].nodes
    nodesXa = allNodes.getByBoundingBox(0-tol, -inf, -inf, 0+tol, inf, inf)
    a.Set(nodes=nodesXa, name='SET-NODES-XA')
    nodesXb = allNodes.getByBoundingBox(dX-tol, -inf, -inf, dX+tol, inf, inf)
    a.Set(nodes=nodesXb, name='SET-NODES-XB')    
    nodesYa = allNodes.getByBoundingBox(-inf, 0-tol, -inf, inf, 0+tol, inf)
    a.Set(nodes=nodesYa, name='SET-NODES-YA')
    nodesYb = allNodes.getByBoundingBox(-inf, dY-tol, -inf, inf, dY+tol, inf)
    a.Set(nodes=nodesYb, name='SET-NODES-YB')
    nodeOrigo=allNodes.getByBoundingBox(0-tol, 0-tol, 0-tol, 0+tol, 0+tol, 0+tol)
    a.Set(nodes=nodeOrigo, name='SET-NODE-ORIGO')
    nodeSubtract=allNodes.getByBoundingBox(dX-tol, dY-tol, 0-tol, dX+tol, dY+tol, 0+tol)
    a.Set(nodes=nodeSubtract, name='SET-NODE-SUB')
    a.SetByBoolean(name='SET-SUB-XB',sets=[a.sets['SET-NODES-XB'],a.sets['SET-NODE-SUB']],operation=DIFFERENCE)
    
def stepInitialBC(modelName,dX,dY,th):
    # Non-linear
    # Here we apply all BC that will be present for any load case:
    m=mdb.models[modelName]
    a=m.rootAssembly
    # Zero z-displacemet on the single node at origo:
    reg = a.sets['SET-NODE-ORIGO']
    m.DisplacementBC(name='BC-Z', createStepName='Initial', 
        region=reg, u1=UNSET, u2=UNSET, u3=0.0, ur1=UNSET, ur2=UNSET, 
        ur3=UNSET, amplitude=UNSET, fixed=OFF, distributionType=UNIFORM, 
        fieldName='', localCsys=None)
    # Zero x-displacement at the surface at x=0:
    reg = a.sets['SET-NODES-XA']
    m.DisplacementBC(name='BC-Xa', createStepName='Initial', 
        region=reg, u1=0.0, u2=UNSET, u3=UNSET, ur1=UNSET, ur2=UNSET, 
        ur3=UNSET, amplitude=UNSET, fixed=OFF, distributionType=UNIFORM, 
        fieldName='', localCsys=None)   
    # Zero y-displacement at the surface at y=0:
    reg = a.sets['SET-NODES-YA']
    m.DisplacementBC(name='BC-Ya', createStepName='Initial', 
        region=reg, u1=UNSET, u2=0.0, u3=UNSET, ur1=UNSET, ur2=UNSET, 
        ur3=UNSET, amplitude=UNSET, fixed=OFF, distributionType=UNIFORM, 
        fieldName='', localCsys=None)
    
    # Linear
    # Here we apply all BC that will be present for any load case:
    modelNameL = modelName + '-L'
    m=mdb.models[modelNameL]
    a=m.rootAssembly
    # Zero z-displacemet on the single node at origo:
    reg = a.sets['SET-NODE-ORIGO']
    m.DisplacementBC(name='BC-Z', createStepName='Initial', 
        region=reg, u1=UNSET, u2=UNSET, u3=0.0, ur1=UNSET, ur2=UNSET, 
        ur3=UNSET, amplitude=UNSET, fixed=OFF, distributionType=UNIFORM, 
        fieldName='', localCsys=None)
    # Zero x-displacement at the surface at x=0:
    reg = a.sets['SET-NODES-XA']
    m.DisplacementBC(name='BC-Xa', createStepName='Initial', 
        region=reg, u1=0.0, u2=UNSET, u3=UNSET, ur1=UNSET, ur2=UNSET, 
        ur3=UNSET, amplitude=UNSET, fixed=OFF, distributionType=UNIFORM, 
        fieldName='', localCsys=None)   
    # Zero y-displacement at the surface at y=0:
    reg = a.sets['SET-NODES-YA']
    m.DisplacementBC(name='BC-Ya', createStepName='Initial', 
        region=reg, u1=UNSET, u2=0.0, u3=UNSET, ur1=UNSET, ur2=UNSET, 
        ur3=UNSET, amplitude=UNSET, fixed=OFF, distributionType=UNIFORM, 
        fieldName='', localCsys=None)            

def makeConstr(modelName):
    # Non-linear
    m=mdb.models[modelName]
    m.Equation(name='VertSurfXB', terms=((1.0, 'SET-SUB-XB', 1), (-1.0, 
        'SET-NODE-SUB', 1)))
    
    # Linear
    modelNameL = modelName + '-L'
    m=mdb.models[modelNameL]
    m.Equation(name='VertSurfXB', terms=((1.0, 'SET-SUB-XB', 1), (-1.0, 
        'SET-NODE-SUB', 1)))
    
def stepsInPlaneY(modelName,dX,dY,th):
    # Non-linear
    m=mdb.models[modelName]
    nonLin = 'Step-InPlaneY-NL'
    
    modelNameL = modelName + '-L'
    mL=mdb.models[modelNameL]
    lin = 'Step-InPlaneY-L'

    # Decide how many steps you want in the variable stepNum
    stepNum = numSteps
    yDisp = (dY*MidPlaneStrain)/stepNum
    inc = yDisp
    for i in range(stepNum):
        ip = str(i+1)
        
        m.StaticStep(name=nonLin+ip, previous=m.steps.keys()[-1], 
            initialInc=0.1, maxInc=0.1, nlgeom=ON)      
        a=m.rootAssembly           
        # Fixed in x-direction:
#        reg = a.sets['SET-NODES-XB']
#        m.DisplacementBC(name='BC-XB-ip-'+ip, createStepName=nonLin+ip, 
#            region=reg, u1=0.0, u2=UNSET, u3=UNSET, ur1=UNSET, ur2=UNSET, 
#            ur3=UNSET, amplitude=UNSET, fixed=OFF, distributionType=UNIFORM, 
#            fieldName='', localCsys=None)
        # Prescribed displacement y-direction:
        reg = a.sets['SET-NODES-YB']
        m.DisplacementBC(name='BC-YB-displ-'+ip, createStepName=nonLin+ip, 
            region=reg, u1=UNSET, u2=yDisp, u3=UNSET, ur1=UNSET, ur2=UNSET, 
            ur3=UNSET, amplitude=UNSET, fixed=OFF, distributionType=UNIFORM, 
            fieldName='', localCsys=None)
        
        
        # Linear
        mL.StaticStep(name=lin+ip, previous=mL.steps.keys()[-1], 
            initialInc=0.1, maxInc=0.1, nlgeom=OFF)
        a=mL.rootAssembly       
        # Fixed in x-direction:
#        reg = a.sets['SET-NODES-XB']
#        mL.DisplacementBC(name='BC-XB-ip'+ip, createStepName=lin+ip, 
#            region=reg, u1=0.0, u2=UNSET, u3=UNSET, ur1=UNSET, ur2=UNSET, 
#            ur3=UNSET, amplitude=UNSET, fixed=OFF, distributionType=UNIFORM, 
#            fieldName='', localCsys=None)
        # Prescribed displacement y-direction:
        reg = a.sets['SET-NODES-YB']
        mL.DisplacementBC(name='BC-YB-displ'+ip, createStepName=lin+ip, 
            region=reg, u1=UNSET, u2=yDisp, u3=UNSET, ur1=UNSET, ur2=UNSET, 
            ur3=UNSET, amplitude=UNSET, fixed=OFF, distributionType=UNIFORM, 
            fieldName='', localCsys=None)
        yDisp = yDisp + inc  
    
def makeJobs(modelName):
    # Non-linear    
    jobName = modelName
    mdb.Job(name=jobName, model=modelName, description='', type=ANALYSIS, atTime=None, 
        waitMinutes=0, waitHours=0, queue=None, memory=90, memoryUnits=PERCENTAGE, 
        getMemoryFromAnalysis=True, explicitPrecision=SINGLE, 
        nodalOutputPrecision=SINGLE, echoPrint=OFF, modelPrint=OFF, 
        contactPrint=OFF, historyPrint=OFF, userSubroutine='', scratch='', 
        resultsFormat=ODB, multiprocessingMode=DEFAULT, numCpus=1, numGPUs=0)
    
    mdb.jobs[jobName].writeInput(consistencyChecking=OFF)
    
    # Linear
    modelNameL = modelName + '-L'
    jobName = modelNameL
    mdb.Job(name=jobName, model=modelNameL, description='', type=ANALYSIS, atTime=None, 
        waitMinutes=0, waitHours=0, queue=None, memory=90, memoryUnits=PERCENTAGE, 
        getMemoryFromAnalysis=True, explicitPrecision=SINGLE, 
        nodalOutputPrecision=SINGLE, echoPrint=OFF, modelPrint=OFF, 
        contactPrint=OFF, historyPrint=OFF, userSubroutine='', scratch='', 
        resultsFormat=ODB, multiprocessingMode=DEFAULT, numCpus=1, numGPUs=0)
    
    mdb.jobs[jobName].writeInput(consistencyChecking=OFF)

    # Make dim for running
def makeDimensions(mdata):               #a,b,c,d
    dim1=float(mdata[4])                 #a
    dim2=float(mdata[5])                 #b
    dim3=float(mdata[6])                 #c
    dim4=float(mdata[7])                 #d
    dim5=float(mdata[8])                 #theta
    th=float(mdata[9])                   #thickness
    return (dim1,dim2,dim3,dim4,dim5,th)

def makeUsefulFile(modelName,dX,dY,th):
    # Non-linear
    fileName = 'usefulFile-'+modelName+'.txt'
    f = open(fileName,'w')
    string = [modelName, '\t', str(dX), '\t', str(dY), '\t', str(th)]
    f.writelines(string)
    f.close()
    
    # Linear
    modelNameL = modelName + '-L'
    fileName = 'usefulFile-'+modelNameL+'.txt'
    f = open(fileName,'w')
    string = [modelNameL, '\t', str(dX), '\t', str(dY), '\t', str(th)]
    f.writelines(string)
    f.close()

# Main program
def runModel(modelName,esize,Emod,Pois,dim1,dim2,dim3,dim4,dim5,thickness):
    modelName = 'pt-YdX-'+modelName
    makeModel(modelName,dim1,dim2,dim3,dim4,dim5,thickness)
    copyBasePartToNewModel(modelName)
    meshAndMirror(modelName,esize)
    materialAndProperties(modelName,Emod,Pois)
    
    dims=getDimensions(modelName)
    dX, dY, th = dims[3], dims[4], dims[5]
    makeUsefulFile(modelName,dX,dY,th)
    createAssemblySets(modelName,dX,dY,th)
    stepInitialBC(modelName,dX,dY,th)
    makeConstr(modelName)
    stepsInPlaneY(modelName,dX,dY,th)
    
    makeJobs(modelName)


# =============================================================================
#   ### GLOBAL PARAMETERS ###
# =============================================================================

numSteps = int(sys.argv[-1])      
dataFileName=sys.argv[-2]
lineStart=int(sys.argv[-4])
lineStop=(lineStart-1) + int(sys.argv[-3])
#workDir = os.getcwd()
start = str(sys.argv[-4])
end = str(lineStop)
dataFile=open(dataFileName)

modelsData=[]
for line in dataFile:
    dataList=[]
    data=line.split()
    for d in data:
        dataList.append(d.strip(' \t\n\r'))
    modelsData.append(dataList) 
    
dataFile.close()

if len(modelsData) < int(sys.argv[-3]):
    print('Too many jobs requested\n')
elif (int(sys.argv[-4])-1) < 0:
    print('Invalid start number!\n')    
elif (len(modelsData)-(lineStart-1)) < int(sys.argv[-3]):
    print('Not enough datalines left to give', int(sys.argv[-3]),'results\n')   
else:
    for i in range(lineStart-1, lineStop):
        mdata = modelsData[i]
        DIMS = makeDimensions(mdata)
        Curvature = float(mdata[11])
        MidPlaneStrain = float(mdata[10])
    
        #    modelName      esize              emod               pois             dim1     dim2     dim3     dim4     dim5     th  
        runModel(mdata[0],  float(mdata[1]),  float(mdata[2]),  float(mdata[3]), DIMS[0], DIMS[1], DIMS[2], DIMS[3], DIMS[4], DIMS[5])

